from datetime import date
from typing_extensions import Literal

EASTER_JULIAN: Literal[1]
EASTER_ORTHODOX: Literal[2]
EASTER_WESTERN: Literal[3]

def easter(year: int, method: Literal[1, 2, 3] = ...) -> date: ...



from typing import Any, Callable, Dict, Generic, Iterator, Optional, TypeVar, Union

_K = TypeVar("_K")
_V = TypeVar("_V")
_T = TypeVar("_T")
_T2 = TypeVar("_T2")

class defaultdict(Dict[_K, _V]):
    default_factory: None
    def __init__(self, __default_factory: Callable[[], _V] = ..., init: Any = ...) -> None: ...
    def __missing__(self, key: _K) -> _V: ...
    def __copy__(self: _T) -> _T: ...
    def copy(self: _T) -> _T: ...

class deque(Generic[_T]):
    maxlen: Optional[int]
    def __init__(self, iterable: Iterator[_T] = ..., maxlen: int = ...) -> None: ...
    def append(self, x: _T) -> None: ...
    def appendleft(self, x: _T) -> None: ...
    def clear(self) -> None: ...
    def count(self, x: Any) -> int: ...
    def extend(self, iterable: Iterator[_T]) -> None: ...
    def extendleft(self, iterable: Iterator[_T]) -> None: ...
    def pop(self) -> _T: ...
    def popleft(self) -> _T: ...
    def remove(self, value: _T) -> None: ...
    def reverse(self) -> None: ...
    def rotate(self, n: int = ...) -> None: ...
    def __contains__(self, o: Any) -> bool: ...
    def __copy__(self) -> deque[_T]: ...
    def __getitem__(self, i: int) -> _T: ...
    def __iadd__(self, other: deque[_T2]) -> deque[Union[_T, _T2]]: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __len__(self) -> int: ...
    def __reversed__(self) -> Iterator[_T]: ...
    def __setitem__(self, i: int, x: _T) -> None: ...



import selectors
from typing import Optional

from . import base_events

class BaseSelectorEventLoop(base_events.BaseEventLoop):
    def __init__(self, selector: Optional[selectors.BaseSelector] = ...) -> None: ...



from typing import Any, Container, Iterable, Optional, Text

from bleach.linkifier import DEFAULT_CALLBACKS as DEFAULT_CALLBACKS, Linker as Linker, _Callback
from bleach.sanitizer import (
    ALLOWED_ATTRIBUTES as ALLOWED_ATTRIBUTES,
    ALLOWED_PROTOCOLS as ALLOWED_PROTOCOLS,
    ALLOWED_STYLES as ALLOWED_STYLES,
    ALLOWED_TAGS as ALLOWED_TAGS,
    Cleaner as Cleaner,
)

__releasedate__: Text
__version__: Text
VERSION: Any  # packaging.version.Version

def clean(
    text: Text,
    tags: Container[Text] = ...,
    attributes: Any = ...,
    styles: Container[Text] = ...,
    protocols: Container[Text] = ...,
    strip: bool = ...,
    strip_comments: bool = ...,
) -> Text: ...
def linkify(
    text: Text, callbacks: Iterable[_Callback] = ..., skip_tags: Optional[Container[Text]] = ..., parse_email: bool = ...
) -> Text: ...



import sys
from typing import Any, Optional, Tuple, List, overload, TypeVar

from django.db.models.base import Model

from django.db import models

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

_T = TypeVar("_T", bound=Model)

class BaseUserManager(models.Manager[_T]):
    @classmethod
    def normalize_email(cls, email: Optional[str]) -> str: ...
    def make_random_password(self, length: int = ..., allowed_chars: str = ...) -> str: ...
    def get_by_natural_key(self, username: Optional[str]) -> _T: ...

class AbstractBaseUser(models.Model):
    REQUIRED_FIELDS: List[str] = ...

    password = models.CharField(max_length=128)
    last_login = models.DateTimeField(blank=True, null=True)
    def get_username(self) -> str: ...
    def natural_key(self) -> Tuple[str]: ...
    @property
    def is_anonymous(self) -> Literal[False]: ...
    @property
    def is_authenticated(self) -> Literal[True]: ...
    def set_password(self, raw_password: Optional[str]) -> None: ...
    def check_password(self, raw_password: str) -> bool: ...
    def set_unusable_password(self) -> None: ...
    def has_usable_password(self) -> bool: ...
    def get_session_auth_hash(self) -> str: ...
    @classmethod
    def get_email_field_name(cls) -> str: ...
    @classmethod
    @overload
    def normalize_username(cls, username: str) -> str: ...
    @classmethod
    @overload
    def normalize_username(cls, username: Any) -> Any: ...



from typing import Any

BATCH_WRITE_ITEM: str
DESCRIBE_TABLE: str
BATCH_GET_ITEM: str
CREATE_TABLE: str
UPDATE_TABLE: str
DELETE_TABLE: str
LIST_TABLES: str
UPDATE_ITEM: str
DELETE_ITEM: str
GET_ITEM: str
PUT_ITEM: str
QUERY: str
SCAN: str
GLOBAL_SECONDARY_INDEX_UPDATES: str
RETURN_ITEM_COLL_METRICS: str
EXCLUSIVE_START_TABLE_NAME: str
RETURN_CONSUMED_CAPACITY: str
COMPARISON_OPERATOR: str
SCAN_INDEX_FORWARD: str
ATTR_DEFINITIONS: str
ATTR_VALUE_LIST: str
TABLE_DESCRIPTION: str
UNPROCESSED_KEYS: str
UNPROCESSED_ITEMS: str
CONSISTENT_READ: str
DELETE_REQUEST: str
RETURN_VALUES: str
REQUEST_ITEMS: str
ATTRS_TO_GET: str
ATTR_UPDATES: str
TABLE_STATUS: str
SCAN_FILTER: str
TABLE_NAME: str
KEY_SCHEMA: str
ATTR_NAME: str
ATTR_TYPE: str
ITEM_COUNT: str
CAMEL_COUNT: str
PUT_REQUEST: str
INDEX_NAME: str
ATTRIBUTES: str
TABLE_KEY: str
RESPONSES: str
RANGE_KEY: str
KEY_TYPE: str
ACTION: str
UPDATE: str
EXISTS: str
SELECT: str
ACTIVE: str
LIMIT: str
ITEMS: str
ITEM: str
KEYS: str
UTC: str
KEY: str
DEFAULT_ENCODING: str
DEFAULT_REGION: str
DATETIME_FORMAT: str
SERVICE_NAME: str
HTTP_OK: int
HTTP_BAD_REQUEST: int
PROVISIONED_THROUGHPUT: str
READ_CAPACITY_UNITS: str
WRITE_CAPACITY_UNITS: str
STRING_SHORT: str
STRING_SET_SHORT: str
NUMBER_SHORT: str
NUMBER_SET_SHORT: str
BINARY_SHORT: str
BINARY_SET_SHORT: str
MAP_SHORT: str
LIST_SHORT: str
BOOLEAN: str
BOOLEAN_SHORT: str
STRING: str
STRING_SET: str
NUMBER: str
NUMBER_SET: str
BINARY: str
BINARY_SET: str
MAP: str
LIST: str
SHORT_ATTR_TYPES: Any
ATTR_TYPE_MAP: Any
LOCAL_SECONDARY_INDEX: str
LOCAL_SECONDARY_INDEXES: str
GLOBAL_SECONDARY_INDEX: str
GLOBAL_SECONDARY_INDEXES: str
PROJECTION: str
PROJECTION_TYPE: str
NON_KEY_ATTRIBUTES: str
KEYS_ONLY: str
ALL: str
INCLUDE: str
STREAM_VIEW_TYPE: str
STREAM_SPECIFICATION: str
STREAM_ENABLED: str
STREAM_NEW_IMAGE: str
STREAM_OLD_IMAGE: str
STREAM_NEW_AND_OLD_IMAGE: str
STREAM_KEYS_ONLY: str
EXCLUSIVE_START_KEY: str
LAST_EVALUATED_KEY: str
QUERY_FILTER: str
BEGINS_WITH: str
BETWEEN: str
EQ: str
NE: str
LE: str
LT: str
GE: str
GT: str
IN: str
KEY_CONDITIONS: str
COMPARISON_OPERATOR_VALUES: Any
QUERY_OPERATOR_MAP: Any
NOT_NULL: str
NULL: str
CONTAINS: str
NOT_CONTAINS: str
ALL_ATTRIBUTES: str
ALL_PROJECTED_ATTRIBUTES: str
SPECIFIC_ATTRIBUTES: str
COUNT: str
SELECT_VALUES: Any
SCAN_OPERATOR_MAP: Any
QUERY_FILTER_OPERATOR_MAP: Any
DELETE_FILTER_OPERATOR_MAP: Any
UPDATE_FILTER_OPERATOR_MAP: Any
PUT_FILTER_OPERATOR_MAP: Any
SEGMENT: str
TOTAL_SEGMENTS: str
SCAN_FILTER_VALUES: Any
QUERY_FILTER_VALUES: Any
DELETE_FILTER_VALUES: Any
VALUE: str
EXPECTED: str
CONSUMED_CAPACITY: str
CAPACITY_UNITS: str
INDEXES: str
TOTAL: str
NONE: str
RETURN_CONSUMED_CAPACITY_VALUES: Any
SIZE: str
RETURN_ITEM_COLL_METRICS_VALUES: Any
ALL_OLD: str
UPDATED_OLD: str
ALL_NEW: str
UPDATED_NEW: str
RETURN_VALUES_VALUES: Any
PUT: str
DELETE: str
ADD: str
ATTR_UPDATE_ACTIONS: Any
BATCH_GET_PAGE_LIMIT: int
BATCH_WRITE_PAGE_LIMIT: int
META_CLASS_NAME: str
REGION: str
HOST: str
CONDITIONAL_OPERATOR: str
AND: str
OR: str
CONDITIONAL_OPERATORS: Any



class AlreadyFinalized(Exception): ...
class AlreadyUpdated(Exception): ...
class InvalidKey(Exception): ...
class InvalidSignature(Exception): ...
class InvalidTag(Exception): ...
class NotYetFinalized(Exception): ...
class UnsupportedAlgorithm(Exception): ...



from typing import Tuple

from . import _LangModelType

KOI8R_char_to_order_map: Tuple[int, ...]
win1251_char_to_order_map: Tuple[int, ...]
latin5_char_to_order_map: Tuple[int, ...]
macCyrillic_char_to_order_map: Tuple[int, ...]
IBM855_char_to_order_map: Tuple[int, ...]
IBM866_char_to_order_map: Tuple[int, ...]
RussianLangModel: Tuple[int, ...]
Koi8rModel: _LangModelType
Win1251CyrillicModel: _LangModelType
Latin5CyrillicModel: _LangModelType
MacCyrillicModel: _LangModelType
Ibm866Model: _LangModelType
Ibm855Model: _LangModelType



from random import Random
from thread import LockType
from typing import IO, Any, AnyStr, Iterable, Iterator, List, Optional, Text, Tuple, Union, overload

TMP_MAX: int
tempdir: str
template: str
_name_sequence: Optional[_RandomNameSequence]

class _RandomNameSequence:
    characters: str = ...
    mutex: LockType
    @property
    def rng(self) -> Random: ...
    def __iter__(self) -> _RandomNameSequence: ...
    def next(self) -> str: ...
    # from os.path:
    def normcase(self, path: AnyStr) -> AnyStr: ...

class _TemporaryFileWrapper(IO[str]):
    delete: bool
    file: IO[str]
    name: Any
    def __init__(self, file: IO[str], name: Any, delete: bool = ...) -> None: ...
    def __del__(self) -> None: ...
    def __enter__(self) -> _TemporaryFileWrapper: ...
    def __exit__(self, exc, value, tb) -> Optional[bool]: ...
    def __getattr__(self, name: unicode) -> Any: ...
    def close(self) -> None: ...
    def unlink(self, path: unicode) -> None: ...
    # These methods don't exist directly on this object, but
    # are delegated to the underlying IO object through __getattr__.
    # We need to add them here so that this class is concrete.
    def __iter__(self) -> Iterator[str]: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def next(self) -> str: ...
    def read(self, n: int = ...) -> str: ...
    def readable(self) -> bool: ...
    def readline(self, limit: int = ...) -> str: ...
    def readlines(self, hint: int = ...) -> List[str]: ...
    def seek(self, offset: int, whence: int = ...) -> int: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...
    def truncate(self, size: Optional[int] = ...) -> int: ...
    def writable(self) -> bool: ...
    def write(self, s: Text) -> int: ...
    def writelines(self, lines: Iterable[str]) -> None: ...

# TODO text files

def TemporaryFile(
    mode: Union[bytes, unicode] = ...,
    bufsize: int = ...,
    suffix: Union[bytes, unicode] = ...,
    prefix: Union[bytes, unicode] = ...,
    dir: Union[bytes, unicode] = ...,
) -> _TemporaryFileWrapper: ...
def NamedTemporaryFile(
    mode: Union[bytes, unicode] = ...,
    bufsize: int = ...,
    suffix: Union[bytes, unicode] = ...,
    prefix: Union[bytes, unicode] = ...,
    dir: Union[bytes, unicode] = ...,
    delete: bool = ...,
) -> _TemporaryFileWrapper: ...
def SpooledTemporaryFile(
    max_size: int = ...,
    mode: Union[bytes, unicode] = ...,
    buffering: int = ...,
    suffix: Union[bytes, unicode] = ...,
    prefix: Union[bytes, unicode] = ...,
    dir: Union[bytes, unicode] = ...,
) -> _TemporaryFileWrapper: ...

class TemporaryDirectory:
    name: Any
    def __init__(
        self, suffix: Union[bytes, unicode] = ..., prefix: Union[bytes, unicode] = ..., dir: Union[bytes, unicode] = ...
    ) -> None: ...
    def cleanup(self) -> None: ...
    def __enter__(self) -> Any: ...  # Can be str or unicode
    def __exit__(self, type, value, traceback) -> None: ...

@overload
def mkstemp() -> Tuple[int, str]: ...
@overload
def mkstemp(suffix: AnyStr = ..., prefix: AnyStr = ..., dir: Optional[AnyStr] = ..., text: bool = ...) -> Tuple[int, AnyStr]: ...
@overload
def mkdtemp() -> str: ...
@overload
def mkdtemp(suffix: AnyStr = ..., prefix: AnyStr = ..., dir: Optional[AnyStr] = ...) -> AnyStr: ...
@overload
def mktemp() -> str: ...
@overload
def mktemp(suffix: AnyStr = ..., prefix: AnyStr = ..., dir: Optional[AnyStr] = ...) -> AnyStr: ...
def gettempdir() -> str: ...
def gettempprefix() -> str: ...
def _candidate_tempdir_list() -> List[str]: ...
def _get_candidate_names() -> Optional[_RandomNameSequence]: ...
def _get_default_tempdir() -> str: ...



from abc import ABCMeta, abstractmethod
from typing import Optional

from cryptography.hazmat.backends.interfaces import HashBackend

class HashAlgorithm(metaclass=ABCMeta):
    digest_size: int
    name: str

class HashContext(metaclass=ABCMeta):
    algorithm: HashAlgorithm
    @abstractmethod
    def copy(self) -> HashContext: ...
    @abstractmethod
    def finalize(self) -> bytes: ...
    @abstractmethod
    def update(self, data: bytes) -> None: ...

class BLAKE2b(HashAlgorithm): ...
class BLAKE2s(HashAlgorithm): ...
class MD5(HashAlgorithm): ...
class SHA1(HashAlgorithm): ...
class SHA224(HashAlgorithm): ...
class SHA256(HashAlgorithm): ...
class SHA384(HashAlgorithm): ...
class SHA3_224(HashAlgorithm): ...
class SHA3_256(HashAlgorithm): ...
class SHA3_384(HashAlgorithm): ...
class SHA3_512(HashAlgorithm): ...
class SHA512(HashAlgorithm): ...
class SHA512_224(HashAlgorithm): ...
class SHA512_256(HashAlgorithm): ...

class SHAKE128(HashAlgorithm):
    def __init__(self, digest_size: int) -> None: ...

class SHAKE256(HashAlgorithm):
    def __init__(self, digest_size: int) -> None: ...

class Hash(HashContext):
    def __init__(self, algorithm: HashAlgorithm, backend: Optional[HashBackend] = ...): ...
    def copy(self) -> Hash: ...
    def finalize(self) -> bytes: ...
    def update(self, data: bytes) -> None: ...
